------------------------------------------------------------------------------
--	FILE:	 TSLEE_Script.lua
--  totalslacker 
------------------------------------------------------------------------------
include "MapUtilities"

print("Loading TSL Earth Remastered starting script")
-- ===========================================================================
-- Globals
-- ===========================================================================

local iTurn 				= Game.GetCurrentGameTurn()
local mapName 				= MapConfiguration.GetValue("MapName")
local bAddGoodies 			= MapConfiguration.GetValue("ExtraTribes")
local iAddGoodies 			= 0
local bNoGoodies 			= GameConfiguration.GetValue("GAME_NO_GOODY_HUTS")
local bConvertMaize			= MapConfiguration.GetValue("ConvertMaize")
local bLeyLines				= MapConfiguration.GetValue("LeyLines")

print("Map Name: "..tostring(mapName))
print("Extra tribes: "..tostring(bAddGoodies))
print("New Frontier Resources: "..tostring(bConvertMaize))
print("Ley Lines: "..tostring(bLeyLines))

-- ===========================================================================
-- Map Configuration Functions
-- ===========================================================================

function GoodyHuts()
	if iAddGoodies < bAddGoodies then
		print("Adding more tribal villages to map")
		local gridWidth, gridHeight = Map.GetGridSize()
		AddGoodies(gridWidth, gridHeight)
		iAddGoodies = iAddGoodies + 1
		print("iAddGoodies = " .. tostring(iAddGoodies))
	end
end

function ConvertMaize()
	if bConvertMaize then
		print("Attempting to add Maize resources to the Americas")
		local bMaize = false
		if GameInfo.Resources['RESOURCE_MAIZE'].Index ~= nil then bMaize = true end
		if not bMaize then 
			print("Maize resource not detected, ending function")
			return 
		end
		print("Maize resource detected")
		local tContinents = Map.GetContinentsInUse()
		for i,iContinent in ipairs(tContinents) do
			local bNorthAmerica = false
			local bSouthAmerica = false
			local bAmericas		= false
			if GameInfo.Continents[iContinent].ContinentType == "CONTINENT_NORTH_AMERICA" then bNorthAmerica = true end
			if GameInfo.Continents[iContinent].ContinentType == "CONTINENT_SOUTH_AMERICA" then bSouthAmerica = true end
			if GameInfo.Continents[iContinent].ContinentType == "CONTINENT_AMERICA" then bAmericas = true end
			if bNorthAmerica or bSouthAmerica or bAmericas then
				print("Converting Wheat bonus resources in the Americas to Maize")
				local continentPlots = Map.GetContinentPlots(iContinent)
				for i, iPlotIndex in ipairs(continentPlots) do
					-- print("Checking continent plot for resource...")
					local pPlot = Map.GetPlotByIndex(iPlotIndex)
					if pPlot:GetResourceType() ~= nil then
						if pPlot:GetResourceType() == GameInfo.Resources['RESOURCE_WHEAT'].Index then
							ResourceBuilder.SetResourceType(pPlot, -1)
							ResourceBuilder.SetResourceType(pPlot, GameInfo.Resources['RESOURCE_MAIZE'].Index, 1)
							print("Wheat bonus resource has been converted to Maize")
						end
					end
				end
			end
		end
		return true
	end
	return false
end

function AddHoney()
	if bConvertMaize then
		print("Attempting to add Honey resources to Europe")
		local bHoney = false
		if GameInfo.Resources['RESOURCE_HONEY'].Index ~= nil then bHoney = true end
		if not bHoney then 
			print("Honey resource not detected, ending function")
			return 
		end
		print("Honey resource detected")
		local tContinents = Map.GetContinentsInUse()
		for i,iContinent in ipairs(tContinents) do
			local bEurope = false
			if GameInfo.Continents[iContinent].ContinentType == "CONTINENT_EUROPE" then bEurope = true end
			if bEurope then
				print("Attempting to place Honey resources in Europe")
				local continentPlots = Map.GetContinentPlots(iContinent)
				for i, iPlotIndex in ipairs(continentPlots) do
					-- print("Checking continent plot for resource...")
					local pPlot = Map.GetPlotByIndex(iPlotIndex)
					local pTerrain = pPlot:GetTerrainType()
					local pFeature  = pPlot:GetFeatureType()
					local bHoneyPlot = false
					local bHoneyFeature = false
					if GameInfo.Terrains[pTerrain].TerrainType == "TERRAIN_GRASS" then bHoneyPlot = true end
					if GameInfo.Terrains[pTerrain].TerrainType == "TERRAIN_PLAINS" then bHoneyPlot = true end
					if pFeature == -1 then
						bHoneyFeature = true
					elseif(GameInfo.Features[pFeature].FeatureType == "FEATURE_MARSH") then
						bHoneyFeature = true	
					elseif(GameInfo.Features[pFeature].FeatureType == "FEATURE_FLOODPLAINS_GRASSLAND") then
						bHoneyFeature = true	
					elseif(GameInfo.Features[pFeature].FeatureType == "FEATURE_FLOODPLAINS_PLAINS") then
						bHoneyFeature = true				
					end
					-- print("bHoneyPlot is "..tostring(bHoneyPlot))
					-- print("pTerrain is "..tostring(pTerrain))
					-- print("pFeature is "..tostring(pFeature))
					if bHoneyPlot and bHoneyFeature and pPlot:GetResourceType() ~= nil then
						if pPlot:GetResourceType() == -1 then
							local iResourceCount = ResourceBuilder.GetAdjacentResourceCount(pPlot)
							-- print("Adjacent resource count: "..tostring(iResourceCount))
							if iResourceCount < 2 then
								print("Found suitable plot for Honey")
								ResourceBuilder.SetResourceType(pPlot, GameInfo.Resources['RESOURCE_HONEY'].Index, 1)
								print("Honey has been placed successfully")
							end
						end
					end
				end
			end
		end
		return true
	end
	return false
end

function AddLeyLines()
	if bLeyLines then
		print("Attempting to add Ley Lines to the map")
		local bSecretSocieties = false
		if GameInfo.Resources['RESOURCE_LEY_LINE'].Index ~= nil then bSecretSocieties = true end
		if not bSecretSocieties then 
			print("Ley Lines resource not detected, ending function")
			return 
		end
		print("Ley Lines resource detected")	
		local tContinents = Map.GetContinentsInUse()
		for i,iContinent in ipairs(tContinents) do
			if iContinent then
				print("Placing Ley Lines on a new continent")
				local continentPlots = Map.GetContinentPlots(iContinent)
				for i, iPlotIndex in ipairs(continentPlots) do
					-- print("Checking continent plot for resource...")
					local pPlot = Map.GetPlotByIndex(iPlotIndex)
					local bResourcePresent = pPlot:GetResourceType()
					local bFeaturePresent  = pPlot:GetFeatureType()
					-- print("Resource Present: "..tostring(bResourcePresent))
					-- print("Feature Present: "..tostring(bFeaturePresent))
					if bResourcePresent == -1 and bFeaturePresent == -1 then
						local iResourceCount = ResourceBuilder.GetAdjacentResourceCount(pPlot)
						-- print("Adjacent resource count: "..tostring(iResourceCount))
						if iResourceCount == 0 then
							print("Found suitable plot for Ley Line")
							ResourceBuilder.SetResourceType(pPlot, GameInfo.Resources['RESOURCE_LEY_LINE'].Index, 1)
							print("Ley line has been placed successfully")
						end
					end
				end
			end
		end
		return true
	end
	return false
end

-- ===========================================================================
-- Main Function
-- ===========================================================================

function TSLEE_MapConfig(iPlayer)
	local player = Players[iPlayer]
	local mapConfigComplete = Game:GetProperty(iPlayer .. "_TSLEE_MapConfig")
	if player:IsHuman() and not mapConfigComplete then 
		print("-----------------------------------------------------------------------")
		print("Executing TSL Earth Remastered map configuration options via lua script")
		Game:SetProperty(iPlayer .. "_TSLEE_MapConfig", 1)
		if bAddGoodies and not bNoGoodies then
			print(" - Checking goody huts...")
			GoodyHuts()
		end
		if bLeyLines then
			print(" - Checking for Secret Societies options...")
			AddLeyLines()
		end
		if bConvertMaize then
			print(" - Checking for New Frontier options...")
			ConvertMaize()
			AddHoney()
		end
		print("-----------------------------------------------------------------------")
	end
end

-- ===========================================================================
-- Initialization
-- ===========================================================================

function Initialize()
	if (mapName == "TSLEE" or mapName == "CordEarth" or mapName == "TinyEarth" or mapName =="CogEarth" or mapName =="VIKING_GiantEarth") then
		GameEvents.PlayerTurnStarted.Add( TSLEE_MapConfig )
	end
end

Initialize()

-- ===========================================================================
-- End
-- ===========================================================================